@@ -559,10 +559,7 @@ fn test_unmarshal_repeat_times_overflow() -> Result<()> {
     let mut reader = Cursor::new(REPEAT_TIMES_OVERFLOW_SDP.as_bytes());
     let result = SessionDescription::unmarshal(&mut reader);
     assert!(result.is_err());
-    assert_eq!(
-        result.unwrap_err().to_string(),
-        Error::SdpInvalidValue("106751991167301d".to_owned()).to_string()
-    );
+    assert!(Error::SdpInvalidValue("106751991167301d".to_owned()).equal(&result.unwrap_err()));
     Ok(())
 }
 
